// Importing necessary functions from the d3 library
import {
  select,
  csv,
  scaleLinear,
  scaleBand,
  max,
  axisLeft,
  axisBottom,
  line,
} from 'd3';

// Selecting the SVG element and getting its width and height
const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');

// Function to render the visualization
const render = (data) => {
  // Defining chart title and axis labels
  const title = 'Sales Data';
  const xValue = (d) => d.Title;
  const xAxisLabel = 'Game Title';
  const yValue = (d) => d.SalesNorthAmerica;
  const yAxisLabel = 'Sales in North America (millions)';

  // Defining margins for the chart
  const margin = {
    top: 60,
    right: 40,
    bottom: 88,
    left: 150,
  };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  // Appending a 'g' element for the main chart area
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  // Scaling functions for x and y axes
  const xScale = scaleBand()
    .domain(data.map(xValue))
    .range([0, innerWidth])
    .padding(0.1);

  const yScale = scaleLinear()
    .domain([0, max(data, yValue)])
    .range([innerHeight, 0])
    .nice();

  // Creating x and y axes
  const xAxis = axisBottom(xScale);
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10);

  // Appending y-axis to the chart
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -90)
    .attr('x', -innerHeight / 5)
    .attr('fill', 'black')
    .attr('transform', `rotate(-90)`)
    .text(yAxisLabel);

  // Appending x-axis to the chart and adjusting its appearance
  const xAxisG = g
    .append('g')
    .attr('transform', `translate(0,${innerHeight})`)
    .call(xAxis)
    .selectAll('.tick text')
    .attr('transform', 'rotate(-45)')
    .style('text-anchor', 'end')
    .attr('font-size', '12px')
    .attr('dy', '0.35em')
    .filter((d, i) => i % Math.ceil(data.length / 30) !== 0)
    .remove();

  xAxisG.select('.domain').remove();

  // Generating the line for the data points
  const lineGenerator = line()
    .x((d) => xScale(xValue(d)) + xScale.bandwidth() / 2)
    .y((d) => yScale(yValue(d)));

  // Drawing the line path
  g.append('path')
    .attr('class', 'line')
    .attr('d', lineGenerator(data));

  // Removing circles from the code

  // Adding chart title
  g.append('text')
    .attr('class', 'title')
    .attr('y', -10)
    .text(title);
};

// Loading data from a CSV file and rendering the visualization
csv(
  'https://gist.githubusercontent.com/Satishnarne/c7851ee5fa12a262ee386489e5f550e9/raw/fe17ae77e556caa44f851d8e7529065fb241a37a/ICE05-DATA.csv',
).then((data) => {
  // Parsing data as numbers
  data.forEach((d) => {
    d.SalesNorthAmerica = +d.SalesNorthAmerica;
  });
  // Calling the render function with the loaded data
  render(data);
});
